# @summary Individual doas.conf(5) rule
#
# Generate a single line for doas
#
# @param order
#  Alphanumerical key to sort this rule in the file
#
# @param action
#  The action to be taken if this rule matches
#
# @param nopass
#  Whether the user is required to enter a password
#
# @param nolog
#  Whether successful command execution is logged to syslogd(8)
#
# @param persist
#  Whether successful authentication persists for some time
#
# @param keepenv
#  Whether environment variables are retained for the new process
#
# @param setenv
#  List of variables to set or remove
#
# @param identity
#  Running user- or groupname to match
#
# @param target
#  Target user allowed or denied to run as
#
# @param cmd
#  Command allowed or denied to run
#
# @param args
#  Exact command arguments
#
# @example
#   doas::rule { 'everything goes for wheel':
#     nopass   => true,
#     keepenv  => true,
#     setenv   => [
#       'HOME',
#     ],
#     identity => ':wheel',
#   }
define doas::rule (
  Variant[String,Integer]      $order    = $title,

  Enum['permit','deny']        $action   = 'permit',

  Boolean                      $nopass   = false,
  Boolean                      $nolog    = false,
  Boolean                      $persist  = false,
  Boolean                      $keepenv  = false,
  Optional[Array[String[1],1]] $setenv   = undef,  # XXX type stronger

  Doas::Identity               $identity = $title,

  Optional[Doas::Target]       $target   = undef,

  Optional[Stdlib::Unixpath]   $cmd      = undef,

  # the empty list means that $cmd must be run without any arguments
  Optional[Array[String]]      $args     = undef,
) {
  include doas

  if $args and ! $cmd {
    # 'doas -C' may interpret the first argument as command
    fail('$args requires $cmd')
  }

  # all tokens in order of https://man.openbsd.org/doas.conf.5#DESCRIPTION
  $all_tokens = [
    $action,

    { 'nopass'  => $nopass,
      'nolog'   => $nolog,
      'persist' => $persist,
      'keepenv' => $keepenv,
    }.delete_values(false).keys(),

    $setenv ? {
      undef   => [],
      default => ['setenv', '{'] + $setenv + ['}'],
    },

    $identity,

    $target ? {
      undef   => [],
      default => ['as', $target],
    },

    $cmd ? {
      undef   => [],
      default => ['cmd', $cmd],
    },

    $args ? {
      undef   => [],
      default => ['args'] + $args,
    },
  ]

  concat_fragment { $title:
    target  => $doas::config_file,
    order   => $order,
    # yield a single, whitespace separated line
    content => $all_tokens.flatten().join(' '),
  }
}
